
MARKET_DATA_FIELDS = (
    'DAILY',
    'TICK',
)

CACHE_FIELDS = (
    'STATS',
    'PEERS',
    'NEWS',
    'FINANCIALS',
    'EARNINGS',
    'DIVIDENDS',
    'COMPANY',
    'DAILY',
    'TICK',
)

ALL_FIELDS = (
    'QUOTE',
    'COMPOSITION',
    'STATS',
    'PEERS',
    'NEWS',
    'FINANCIALS',
    'EARNINGS',
    'DIVIDENDS',
    'COMPANY',
    'DAILY',
    'TICK',
)


OTHERS = (
    'POPULARETFS'
)


ETF_URL = 'https://etfdailynews.com/etf/%s/'
POPULAR_ETFS_URL = 'http://etfdb.com/compare/volume/'
